@@ -73,14 +73,14 @@ $cmd_chfn     = &Utils::File::locate_tool ("chfn");
 $cmd_pw       = &Utils::File::locate_tool ("pw");
 
 # enum like for verbose group array positions
-my $ID      = 0;
-my $LOGIN   = 1;
-my $PASSWD  = 2;
-my $UID     = 3;
-my $GID     = 4;
-my $COMMENT = 5;
-my $HOME    = 6;
-my $SHELL   = 7;
+my $i = 0;
+my $LOGIN   = $i++;
+my $PASSWD  = $i++;
+my $UID     = $i++;
+my $GID     = $i++;
+my $COMMENT = $i++;
+my $HOME    = $i++;
+my $SHELL   = $i++;
 
 %login_defs_prop_map = ();
 %profiles_prop_map = ();
@@ -376,10 +376,9 @@ sub get_logindefs
 sub get
 {
   my ($ifh, @users, %users_hash);
-  my (@line, @users, $counter);
+  my (@line, @users);
 
   # Find the passwd file.
-  $counter = 1;
   $ifh = &Utils::File::open_read_from_names(@passwd_names);
   return unless ($ifh);
 
@@ -390,9 +389,6 @@ sub get
     next if &Utils::Util::ignore_line ($_);
 
     @line  = split ':', $_, -1;
-    unshift @line, $counter;
-    $counter++;
-
     $login = $line[$LOGIN];
     @comment = split ',', $line[$COMMENT], 5;
 
@@ -440,17 +436,6 @@ sub get
   return \@users;
 }
 
-sub get_files
-{
-  my @arr;
-
-  push @arr, @passwd_names;
-  push @arr, @shadow_names;
-
-  return \@arr;
-}
-
-
 sub del_user
 {
 	my ($user) = @_;
@@ -527,6 +512,32 @@ sub modify_shadow_password
   &Utils::File::save_buffer ($buffer, @shadow_names);
 }
 
+sub set_passwd
+{
+  my ($login, $password) = @_;
+  my ($pwdpipe, $command);
+
+  if ($Utils::Backend::tool{"system"} eq "FreeBSD")
+  {
+
+    $command = "$cmd_pw usermod -H 0";
+    $pwdpipe = &Utils::File::run_pipe_write ($command);
+    print $pwdpipe $password;
+    &Utils::File::close_file ($pwdpipe);
+  }
+  elsif ($Utils::Backend::tool{"system"} eq "SunOS")
+  {
+    &modify_shadow_password ($login, $password);
+  }
+  else
+  {
+    $command = "$cmd_usermod " .
+        " -p '" . $password . "' " . $login;
+
+    &Utils::File::run ($command);
+  }
+}
+
 sub add_user
 {
 	my ($user) = @_;
@@ -580,7 +591,8 @@ sub add_user
     if ($cmd_adduser &&
         $Utils::Backend::tool{"platform"} !~ /^slackware/ &&
         $Utils::Backend::tool{"platform"} !~ /^archlinux/ &&
-        $Utils::Backend::tool{"platform"} !~ /^redhat/)
+        $Utils::Backend::tool{"platform"} !~ /^redhat/ &&
+        $Utils::Backend::tool{"platform"} !~ /^gentoo/)
     {
       # use adduser if available and valid (slackware one is b0rk)
       # set empty gecos fields and password, they will be filled out later
@@ -718,14 +730,14 @@ sub set
 
     foreach $i (@$config) 
     {
-      $users{$$i[0]} |= 1;
-      $config_hash{$$i[0]} = $i;
+      $users{$$i[$LOGIN]} |= 1;
+      $config_hash{$$i[$LOGIN]} = $i;
     }
 	
     foreach $i (@$old_config)
     {
-      $users{$$i[0]} |= 2;
-      $old_config_hash{$$i[0]} = $i;
+      $users{$$i[$LOGIN]} |= 2;
+      $old_config_hash{$$i[$LOGIN]} = $i;
     }
 
     # Delete all groups that only appeared in the old configuration
@@ -735,12 +747,12 @@ sub set
 
       if ($state == 1)
       {
-        # Groups with state 1 have been added to the config
+        # Users with state 1 have been added to the config
         &add_user ($config_hash{$i});
       }
       elsif ($state == 2)
       {
-        # Groups with state 2 have been deleted from the config
+        # Users with state 2 have been deleted from the config
         &del_user ($old_config_hash{$i});
       }
       elsif (($state == 3) &&
@@ -752,4 +764,44 @@ sub set
   }
 }
 
+sub get_user
+{
+  my ($uid) = @_;
+  my ($users) = &get ();
+
+  foreach $user (@$users)
+  {
+    next if ($uid != $$user[$UID]);
+    return ($$user[$UID], $$user[$PASSWD], $$user[$COMMENT]);
+  }
+
+  return ($uid, "", []);
+}
+
+sub set_user
+{
+  my ($uid, $passwd, @comment) = @_;
+  my ($users) = &get ();
+
+  foreach $user (@$users)
+  {
+    if ($uid == $$user[$UID])
+    {
+      &set_passwd ($$user[$LOGIN], $passwd);
+      &change_user_chfn ($$user[$LOGIN], undef, @comment);
+      return;
+    }
+  }
+}
+
+sub get_files
+{
+  my ($arr);
+
+  push @$arr, @passwd_names;
+  push @$arr, @shadow_names;
+
+  return $arr;
+}
+
 1;
